/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

const LEVELS = [
    {
        // Angezeigter Arbeitsauftrag (HTML erlaubt)
        task: '<p>Der Roboter erwartet hohen Besuch und will dafür einen roten Teppich ausrollen. Der Teppich soll auf seinem aktuellen Feld beginnen, in Blickrichtung verlaufen, aus einem Plättchen je Feld bestehen und genau zehn Felder lang sein.</p><p>Schreibe einen geeigneten Algorithmus dafür. Verwende <strong>nur vordefinierte Anweisungen und Sequenzen</strong>.</p>',

        // Welt und Roboterstartposition als Maschinencode
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',

        // Quelltext, der anfangs im Editor steht
        code: '# Zehn Felder in Blickrichtung mit je einem Plättchen belegen\n',

        // Ist das Bearbeiten des Quelltextes gesperrt? (Optional, Standardwert false)
        readonly: false,

        // Muss - ggf. zusätzlich zur korrekten Fragenbeantwortung - checkCode oder checkTermination unlock=true liefern, bevor Level abgeschlossen werden kann? (Optional, Standardwert false)
        locked: true,

        // Optional: Überprüft, ob ein syntaktisch korrekter Quellcode ausgeführt werden darf; Rückgabewert true für ja, sonst { accept: "Ausführung erlauben?", line: "Zeilennummer eines Fehlers (optional)", msg: "Fehlermeldung (optional)", fine: "Als einen Fehlversuch werten? (optional)", unlock: "Level als abgeschlossen behandeln? (optional)" }
        checkCode: [
            CodeCheckers.allowOnlyCommands(1),
            CodeCheckers.accept()
        ],

        // Optional: Reagiert auf ein Ende der Algorithmusausführung; Rückgabewert wie checkCode
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.worldCheck(1, [0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0]),
            CodeCheckers.accept(true)
        ],

        // Wird falsches Beantworten der Frage als Fehlversuch gewertet? (Optional, Standardwert false)
        questionWrongFine: 0,

        // Falls erst eine Frage richtig beantwortet werden muss: Welche?
        question: null
    },
    {
        task: 'Dieser Algorithmus bewirkt das Gleiche wie dein Algorithmus. Probiere ihn aus und schaue ihn dir genau an: Was ist der entscheidende Unterschied?',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Zehn Felder in Blickrichtung mit je einem Plättchen belegen\nWIEDERHOLE 10 MAL\n    plus\n    vor\n',
        readonly: true,
        locked: true,
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 1,
        question: {
            html: "Was ist hier anders?",
            choices: [
                { html: "Es gibt ", unit: " neue(s) Schlüsselwort(e)", correct: true, type: "number", min: 1, step: 1, accept: 2, "response": "Neu sind die beiden Schlüsselwörter <code><strong>WIEDERHOLE</strong></code> und <code><strong>MAL</strong></code>." },
                { html: "Es gibt ", unit: " neue vordefinierte Anweisung(en)", correct: false, type: "number", min: 1, step: 1, response: "Bei <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> kann es sich nicht um vordefinierte Anweisungen handeln, weil je Zeile nur eine vordefinierte Anweisung erlaubt ist." },
                { html: "Es gibt ", unit: " neue selbstdefinierte Anweisung(en)", correct: false, type: "number", min: 1, step: 1, response: "Selbstdefinierte Anweisungen beginnen mit dem Schlüsselwort <code><strong>LERNE</strong></code> &dash; und das steht nirgends im Algorithmus. Daher kann es hier keine selbstdefinierte Anweisung geben." },
                { html: "Nichts ist anders" },
                { html: "Die Darstellung des Algorithmus ist anders als sonst" }
            ]
        }
    },
    {
        task: 'Dieser Algorithmus sieht fast genauso aus wie der vorherige. Probiere ihn mal aus. Dann kannst du sicherlich auch erklären, was genau <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> bewirkt.',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Wieder zehn Felder in Blickrichtung mit je einem Plättchen belegen?\nWIEDERHOLE 10 MAL\n    plus\nvor\n',
        readonly: true,
        locked: true,
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 1,
        question: {
            html: "Was genau bewirkt <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code>?",
            choices: [
                { html: "Der gesamte Algorithmus wird ", unit: "-fach ausgeführt", correct: false, type: "number", min: 0, step: 1, accept: 10 },
                { html: "Alles nach <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> wird ", unit: "-fach ausgeführt", correct: false, type: "number", min: 0, step: 1, accept: 10 },
                { html: "Es wird immer die eine Anweisung nach <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> ", unit: "-fach ausgeführt", correct: false, type: "number", min: 0, step: 1, accept: 10 },
                { html: "Es wird immer die auf <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> folgende, weiter eingerückte Sequenz ", unit: "-fach ausgeführt", correct: true, type: "number", min: 0, step: 1, accept: 10 }
            ]
        }
    },
    {
        task: 'Hast du wirklich verstanden, warum das Einrücken wichtig ist? Wenn ja, kannst du doch sicherlich ohne Ausprobieren beantworten, wie viele Plättchen der Roboter bei diesem Algorithmus legt:',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: 'WIEDERHOLE 10 MAL\nplus',
        readonly: true,
        checkCode: [
            CodeCheckers.reject(0, 'Beantworte die Frage ohne den Algorithmus zu testen!')
        ],
        questionWrongFine: 1,
        question: {
            html: "Wie viele Plättchen legt der Roboter beim Ausführen von obigem Algorithmus?",
            choices: [
                { html: "Der Algorithmus führt zu einer Notabschaltung" },
                { html: "Er legt ", unit: " Plättchen", correct: true, type: "number", min: 0, step: 1, accept: 1, response: "Er wiederholt zehnmal alles, was direkt nach <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> steht und eingerückt ist &dash; also wiederholt er zehnmal nichts. Anschließend führt er das eine mal <code>plus</code> aus und legt ein einziges Plättchen." }
            ]
        }
    },
    {
        task: 'Er legt wirklich nur ein Plättchen, weil er zehnmal eine leere Sequenz &dash; also nichts &dash; wiederholt. Wirklich. Probiere es aus und verfolge die schrittweise Ausführung:',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: 'WIEDERHOLE 10 MAL\nplus',
        readonly: true,
        locked: true,
        checkTermination: [ CodeCheckers.accept(true) ],
        questionWrongFine: 1,
        question: {
            html: "Woran erkennst du bei der schrittweisen Ausführung, dass der Roboter wirklich zehnmal nichts macht?",
            singlechoice: true,
            choices: [
                { html: "Er dreht sich zehnmal im Kreis" },
                { html: "Die Hervorhebung durch blauen Hintergrund, welche Zeile aktiv ist, bleibt etwa zehnmal so lange auf <code><strong>WIEDERHOLE</strong> 10 <strong>MAL</strong></code> wie bei <code>vor</code>", correct: true },
                { html: "Die Hervorhebung durch blauen Hintergrund, welche Zeile aktiv ist, verschwindet zehmal kurz" },
                { html: "Das erkennt man gar nicht" }
            ]
        }
    },
    {
        task: 'Das ist wieder der Algorithmus zum legen des roten Teppichs. Ändere ihn so ab, dass der Teppich nur sieben Felder lang ist, überprüfe das Ergebnis und beantworte dann die Frage:',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Zehn Felder in Blickrichtung mit je einem Plättchen belegen\nWIEDERHOLE 10 MAL\n    plus\n    vor\n',
        locked: true,
        checkCode: [
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.worldCheck(1, [0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1]),
            CodeCheckers.ensureRepeat(1, 7, 'plus\nvor'),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 1,
        question: {
            html: "Was sind Vorteile von solchen Wiederholungen mit fester Anzahl?",
            choices: [
                { html: "Der Algorithmus kann kürzer sein", correct: true },
                { html: "Der Algorithmus kann so geschrieben werden, dass ihn Menschen leichter verstehen können", correct: true },
                { html: "Änderungen sind leichter möglich", correct: true },
                { html: "Gleiche Teile brauchen nicht mehrfach beschrieben zu werden", correct: true },
                { html: "Der Roboter kann Probleme lösen, die er ohne Wiederholungen mit fester Anzahl nicht bewältigen kann" }
            ]
        }
    },
    {
        task: 'Nachdem er einen sieben Felder langen roten Teppich ausgerollt hat, soll sich der Roboter umdrehen. Ändere den Algorithmus entsprechend ab und teste ihn.',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Sieben Felder in Blickrichtung mit je einem Plättchen belegen\nWIEDERHOLE 7 MAL\n    plus\n    vor\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.worldCheck(1, [0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1]),
            CodeCheckers.ensureRobotDirection(1, 2),
            CodeCheckers.ensureRepeat(1, 7),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Der sieben Felder lange Teppich soll nicht auf dem Feld beginnen, auf dem der Roboter anfangs steht. Es sollen stattdessen zwei Felder (in Blickrichtung) zum Rand frei sein.',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Sieben Felder in Blickrichtung mit je einem Plättchen belegen\nWIEDERHOLE 7 MAL\n    plus\n    vor\nlinksrum\nlinksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.worldCheck(1, [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1]),
            CodeCheckers.ensureRepeat(1, 7),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Schaffst du es auch, dass der rote Teppich so lange ist, wie die ganze Roboterwelt? Der Teppich soll also an der südlichen Kante beginnen und bis zur nördlichen Kante reichen. Verwende natürlich weiterhin eine Wiederholung mit fester Anzahl',
        world: '013C000000000000000000000000000000000000000000000000000000000000000000000000100',
        code: '# Sieben Felder in Blickrichtung mit je einem Plättchen belegen\nWIEDERHOLE 7 MAL\n    plus\n    vor\nlinksrum\nlinksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIf(),
            CodeCheckers.disallowWhile(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.worldCheck(1, [0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1]),
            CodeCheckers.ensureRepeat(1),
            CodeCheckers.accept(true)
        ]
    }
];

window.roboterCodingTask = {
    levels: LEVELS
};
})(window);
